@@ -3,221 +3,21 @@ File in charge of emulating a basic tty to universalise the interface
 """
 import os
 import sys
-import stat
-import time
-import locale
 import shutil
+import prompt_toolkit
+from typing import List, Dict
+from prompt_toolkit.key_binding import KeyBindings
+from prompt_toolkit.history import InMemoryHistory
 from prettytable import PrettyTable
 from ask_question import AskQuestion
 from colourise_output import ColouriseOutput
-
-
-class HLLs:
-    """
-    The basics of the ls function
-    This code is borrowed from: https://github.com/connormason/lspython/tree/master
-    I :
-        - fixed this code's errors
-        - made it cross-platform
-        - implemented it into the program
-        - adapted the code to fit into the shell's functionalities
-    """
-
-    def __init__(self, success: int = 0, error: int = 84) -> None:
-        # ---- The colours for the TUI ----
-        self.colors = {
-            "default": "",
-            "white": "\x1b[01;37m",
-            "gray": "\x1b[00;37m",
-            "purple": "\x1b[00;35m",
-            "cyan": "\x1b[01;36m",
-            "green": "\x1b[01;32m",
-            "red": "\x1b[01;05;37;41m"
-        }
-        # ---- The status code ----
-        self.success = success
-        self.error = error
-
-    def has_colors(self, stream) -> bool:
-        """ Check if the ncurse library is present in the system for the colour management """
-        if not hasattr(stream, "isatty"):
-            return False
-        if not stream.isatty():
-            return False
-        return False
-
-    def get_mode_info(self, mode, filename):
-        """ Get the type of document in order to apply some colour """
-        perms = "-"
-        color = "default"
-        link = ""
-
-        if stat.S_ISDIR(mode):
-            perms = "d"
-            color = "cyan"
-        elif stat.S_ISLNK(mode):
-            perms = "l"
-            color = "purple"
-            link = os.readlink(filename)
-            if not os.path.exists(filename):
-                color = "red"
-        elif stat.S_ISREG(mode):
-            if mode & (stat.S_IXGRP | stat.S_IXUSR | stat.S_IXOTH):
-                color = "green"
-            else:
-                if filename[0] == '.':
-                    color = "gray"
-                else:
-                    color = "white"
-
-        mode = stat.S_IMODE(mode)
-
-        for who in "USR", "GRP", "OTH":
-            for what in "R", "W", "X":
-                if mode & getattr(stat, "S_I" + what + who):
-                    perms = perms + what.lower()
-                else:
-                    perms = perms + "-"
-
-        return (perms, color, link)
-
-    def get_user_info(self, uid) -> str:
-        """ Get the info of the user """
-        try:
-            import pwd
-            user_info = pwd.getpwuid(uid)
-            return user_info.pw_name
-        except ImportError:
-            return str(uid)
-
-    def get_group_info(self, gid) -> str:
-        """ Get the pid of the active groupe """
-        try:
-            import grp
-            group_info = grp.getgrgid(gid)
-            return group_info.gr_name
-        except ImportError:
-            return str(gid)
-
-    def list_files(self, files: list) -> int:
-        """ List the files contained in the path """
-        global_status = self.success
-        table = PrettyTable(
-            [
-                "Permissions",
-                "# Links",
-                "Owner",
-                "Group",
-                "Size",
-                "Last Mod",
-                "Name"
-            ]
-        )
-
-        locale.setlocale(locale.LC_ALL, '')
-        files.sort(key=lambda x: x.lower())
-
-        now = int(time.time())
-        recent = now - (6 * 30 * 24 * 60 * 60)
-
-        does_have_colors = self.has_colors(sys.stdout)
-
-        for filename in files:
-            try:
-                stat_info = os.lstat(filename)
-            except OSError:
-                sys.stderr.write(f"{filename}: No such file or directory\n")
-                global_status = self.error
-                continue
-
-            perms, color, link = self.get_mode_info(
-                stat_info.st_mode,
-                filename
-            )
-
-            nlink = f"{stat_info.st_nlink:4d}%4d"
-            name = self.get_user_info(stat_info.st_uid)
-            group = self.get_group_info(stat_info.st_gid)
-            size = f"{stat_info.st_size:8d}"
-
-            time_stamp = stat_info.st_mtime
-            if (time_stamp < recent) or (time_stamp > now):
-                time_fmt = "%b %e  %Y"
-            else:
-                time_fmt = "%b %e %R"
-            time_str = time.strftime(time_fmt, time.gmtime(time_stamp))
-
-            if self.colors[color] and does_have_colors:
-                filename_str = self.colors[color] + filename + "\x1b[00m"
-            else:
-                filename_str = filename
-
-            if link:
-                filename_str += " -> "
-            filename_str += link
-
-            table.add_row(
-                [
-                    perms,
-                    nlink,
-                    name,
-                    group,
-                    size,
-                    time_str,
-                    filename_str
-                ]
-            )
-
-        table.align["Permissions"] = 'l'
-        table.align["# Links"] = 'r'
-        table.align["Owner"] = 'l'
-        table.align["Group"] = 'l'
-        table.align["Size"] = 'r'
-        table.align["Last Mod"] = 'l'
-        table.align["Name"] = 'l'
-        print(table)
-        return global_status
-
-    def ls(self, path: str or list = "") -> int:
-        """ 
-        A basic loop manager to make this P.O.S POC a minimum functional and feel like the core of the real ls
-        """
-        try:
-            if path in ("", "."):
-                content = os.listdir(".")
-                return self.list_files(content)
-            if ".." in path:
-                tmp = os.getcwd()
-                os.chdir(path)
-                content = os.listdir(".")
-                status = self.list_files(content)
-                os.chdir(tmp)
-                return status
-            if isinstance(path, list):
-                global_status = self.success
-                for item in path:
-                    print(f"Content of: {item}")
-                    content = [item]
-                    if os.path.isdir(item):
-                        content = os.listdir(item)
-                    status = self.list_files(content)
-                    if status != self.success:
-                        global_status = self.error
-                return global_status
-            if os.path.isdir(path):
-                files = os.listdir(path)
-                return self.list_files(files)
-            files = [path]
-            return self.list_files(files)
-        except Exception as err:
-            print(f"The pseudo Ls has crashed: {err}")
-            return self.error
+from .hl_ls import HLLs
 
 
 class TTY:
     """ The class in charge of simulating a tty """
 
-    def __init__(self, err: int, error: int, success: int, colour_lib: ColouriseOutput, ask_question: AskQuestion, colours: dict, colourise_output: bool = True) -> None:
+    def __init__(self, err: int, error: int, success: int, colour_lib: ColouriseOutput, ask_question: AskQuestion, colours: Dict, colourise_output: bool = True) -> None:
         # ---- The version of the program ----
         self.__version__ = "1.0.0"
         # ---- TTY general info ----
@@ -226,6 +26,8 @@ class TTY:
         self.program_author = "(c) Henry Letellier"
         # ---- Command history ----
         self.history = []
+        self.prompt_history = InMemoryHistory()
+        self.history_index = 0
         # ---- The status codes ----
         self.success = success
         self.err = err
@@ -250,9 +52,16 @@ class TTY:
         self.info_colour = None
         # ---- User input tracking ----
         self.user_input = ""
+        self.user_session = prompt_toolkit.PromptSession()
         self.input_split_char = " "
         # ---- function requirering help from the help function ----
         self.help_function_child_name = "help"
+        self.help_help_options = [
+            "help", "man", ".help",
+            ".h", "/?", "-h",
+            "--h", "-help", "--help"
+        ]
+        self.enable_multi_command_help = True
         # ---- Help colour ----
         self.help_title_colour = None
         self.help_command_colour = None
@@ -279,6 +88,13 @@ class TTY:
         self.comment_token = "--"
         # ---- Pipe input ----
         self.pipe_input = None
+        # ---- Working on the auto-complete functionalities ----
+        self.auto_complete_list = []
+        self.auto_complete_index = 0
+        self.auto_complete_usr_input = ""
+        self.auto_complete_default_usr_input = ""
+        # ---- TTY command description token inner ----
+        self.command_description_token_inner = "desc"
 
     def print_on_tty(self, colour: str, string: str) -> None:
         """ The function in charge of displaying a string on the tty """
@@ -294,7 +110,7 @@ class TTY:
         except IOError:
             return self.error
 
-    def list_to_str(self, hl_list: list[any], join: str = " ") -> str:
+    def list_to_str(self, hl_list: List[any], join: str = " ") -> str:
         """ Convert a list to a string """
         res = ""
         list_length = len(hl_list)-1
@@ -304,7 +120,7 @@ class TTY:
                 res += join
         return res
 
-    def version(self, args: list) -> int:
+    def version(self, args: List) -> int:
         """ Display the version of the program """
         func_name = "version"
         if self.help_function_child_name == func_name:
@@ -324,7 +140,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def show_history(self, args: list) -> int:
+    def show_history(self, args: List) -> int:
         """ Display the history of the commands """
         func_name = "history"
         if self.help_function_child_name == func_name:
@@ -354,7 +170,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def process_session_name(self, args: list) -> int:
+    def process_session_name(self, args: List) -> int:
         """ Change the name of the current session """
         func_name = "session_name"
         if self.help_function_child_name == func_name:
@@ -392,7 +208,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def author(self, args: list) -> int:
+    def author(self, args: List) -> int:
         """ Display the author of the program """
         func_name = "author"
         if self.help_function_child_name == func_name:
@@ -412,7 +228,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def client(self, args: list) -> int:
+    def client(self, args: List) -> int:
         """ Display the client of the program """
         func_name = "client"
         if self.help_function_child_name == func_name:
@@ -435,7 +251,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def env(self, args: list) -> int:
+    def env(self, args: List) -> int:
         """ Display the environement variables """
         func_name = "env"
         if self.help_function_child_name == func_name:
@@ -464,7 +280,7 @@ Output:
         self.print_on_tty(self.success_colour, f"{self.is_admin()}\n")
         self.current_tty_status = self.success
 
-    def env_plus_plus(self, args: list) -> int:
+    def env_plus_plus(self, args: List) -> int:
         """ Display the environement variables """
         func_name = "env++"
         if self.help_function_child_name == func_name:
@@ -488,7 +304,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def setenv(self, args: list) -> int:
+    def setenv(self, args: List) -> int:
         """ Function in charge of setting a variable in the environement of the shell """
         func_name = "setenv"
         if self.help_function_child_name == func_name:
@@ -572,7 +388,7 @@ Output:
         self.current_tty_status = self.err
         return self.err
 
-    def unsetenv(self, args: list) -> int:
+    def unsetenv(self, args: List) -> int:
         """ Function in charge of unsetting a variable in the environement of the shell """
         func_name = "unsetenv"
         if self.help_function_child_name == func_name:
@@ -656,7 +472,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def display_status_code(self, args: list) -> None:
+    def display_status_code(self, args: List) -> None:
         """ Display the status code of the last function """
         func_name = "?"
         if self.help_function_child_name == func_name:
@@ -759,10 +575,10 @@ Here are the different status colours:
         )
         self.current_tty_status = self.success
 
-    def process_help_call(self, args: list) -> int:
+    def process_help_call(self, args: List) -> int:
         """ Process the inputs for the help calls """
         usr_input = args[0].lower()
-        if usr_input in ("help", "man", ".help", ".h", "/?", "-h", "--h", "-help", "--help"):
+        if usr_input in self.help_help_options:
             self.help_help()
             self.current_tty_status = self.success
             return self.success
@@ -771,19 +587,32 @@ Here are the different status colours:
             self.current_tty_status = self.success
             return self.current_tty_status
         for item in self.options:
-            if usr_input in item:
+            if usr_input == list(item)[0]:
                 self.help_function_child_name = usr_input
-                item[usr_input](args)
+                item[usr_input](args[1:])
                 self.current_tty_status = self.success
                 return self.current_tty_status
-        self.print_on_tty(self.error_colour, "Invalid option\n")
+        self.print_on_tty(
+            self.error_colour,
+            f"Invalid option: {str(args[0])}\n"
+        )
         self.current_tty_status = self.error
         return self.current_tty_status
 
-    def help(self, args: list) -> int:
+    def help(self, args: List) -> int:
         """ The help function in charge of displaying the available options to the user """
-        if len(args) > 0 and args[0] != '':
-            return self.process_help_call(args)
+        argsc = len(args)
+        if argsc > 0 and args[0] != '':
+            if argsc > 1 and self.enable_multi_command_help is True:
+                global_status = self.success
+                for i in enumerate(args):
+                    status = self.process_help_call(args[i[0]:])
+                    if status != self.success:
+                        global_status = self.error
+            else:
+                global_status = self.process_help_call(args)
+            self.current_tty_status = global_status
+            return global_status
         self.print_on_tty(self.reset_colour, "Available commands:\n")
         if self.options is None:
             self.print_on_tty(self.reset_colour, "No commands available")
@@ -803,7 +632,7 @@ Here are the different status colours:
         self.current_tty_status = self.success
         return self.current_tty_status
 
-    def pwd(self, args: list) -> int:
+    def pwd(self, args: List) -> int:
         """ The function in charge of displaying the current working directory """
         func_name = "pwd"
         if self.help_function_child_name == func_name:
@@ -869,7 +698,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def make_directory(self, args: list) -> int:
+    def make_directory(self, args: List) -> int:
         """ Create a directory """
         func_name = "mkdir"
         if self.help_function_child_name == func_name:
@@ -986,7 +815,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def touch(self, arg: list) -> int:
+    def touch(self, arg: List) -> int:
         """ Create a file in the present path """
         func_name = "touch"
         if self.help_function_child_name == func_name:
@@ -1067,7 +896,7 @@ Output:
             return self.success
         return self.error
 
-    def remove_directory(self, args: list) -> int:
+    def remove_directory(self, args: List) -> int:
         """ Remove a directory """
         func_name = "rmdir"
         if self.help_function_child_name == func_name:
@@ -1161,7 +990,7 @@ Output:
             self.current_tty_status = self.err
             return self.error
 
-    def remove_file(self, args: list) -> int:
+    def remove_file(self, args: List) -> int:
         """ Remove a file or a directory """
         func_name = "rm"
         if self.help_function_child_name == func_name:
@@ -1238,7 +1067,7 @@ Output:
             self.current_tty_status = self.success
             return self.success
         except IOError:
-            self.print_on_tty(self.error_colour, "Invalid path")
+            self.print_on_tty(self.error_colour, f"Invalid path: {str(path)}")
             self.current_tty_status = self.error
             return self.error
 
@@ -1266,7 +1095,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def change_directory(self, args: list) -> int:
+    def change_directory(self, args: List) -> int:
         """ The function in charge of changing the current working directory """
         func_name = "cd"
         if self.help_function_child_name == func_name:
@@ -1310,7 +1139,7 @@ Output:
         self.current_tty_status = self.error
         return self.error
 
-    def exit(self, args: list) -> int:
+    def exit(self, args: List) -> int:
         """ The function in charge of exiting the layer in which the user is located """
         func_name = "exit"
         if self.help_function_child_name == func_name:
@@ -1332,7 +1161,7 @@ Output:
         self.continue_tty_loop = False
         return self.current_tty_status
 
-    def kill(self, args: list) -> int:
+    def kill(self, args: List) -> int:
         """ The function in charge of abruptly stopping the program (not recommended) """
         func_name = "abort"
         if self.help_function_child_name == func_name:
@@ -1361,20 +1190,55 @@ Output:
             self.print_on_tty(self.error_colour, "~")
         self.print_on_tty(self.default_colour, " ")
 
+    def create_key_prompt_bindings(self) -> None:
+        """ Set up the functions in charge of changing the prompt with the content of the history command """
+        bindings = KeyBindings()
+
+        @bindings.add('up')
+        def _(event):
+            if self.history_index > 0:
+                self.history_index -= 1
+                self.user_session.default_buffer.text = self.history[self.history_index]
+
+        @bindings.add('down')
+        def _(event):
+            if self.history_index < len(self.history):
+                self.history_index += 1
+                if self.history_index == len(self.history):
+                    self.user_session.default_buffer.reset()
+                else:
+                    self.user_session.default_buffer.text = self.history[self.history_index]
+
+        self.user_session = prompt_toolkit.PromptSession(
+            complete_while_typing=True,
+            validate_while_typing=True,
+            enable_history_search=True,
+            key_bindings=bindings,
+            history=self.history
+        )
+
+    def process_key_inputs(self) -> str:
+        """ act depending on the special keys pressed or if entered is pressed """
+        try:
+            self.user_input = self.user_session.prompt()
+        except KeyboardInterrupt:
+            self.user_input = ""
+        return self.user_input
+
     def display_prompt(self) -> None:
         """ The function in charge of displaying a basic prompt to ask the user to enter an option """
         self.display_status_in_prompt()
         self.print_on_tty(self.prompt_colour, "(")
         self.print_on_tty(self.session_name_colour, f"{self.session_name}")
         self.print_on_tty(self.prompt_colour, f") {os.getcwd()}>")
-        self.user_input = input("")
+        self.user_input = self.process_key_inputs()
 
     def get_current_folder(self) -> str:
         """ Return the current folder """
         path = os.getcwd()
         path = path.replace("\\", "/")
 
-    def bind_ls(self, args: list) -> int:
+    def bind_ls(self, args: List) -> int:
         """ Bind the ls function to the ls command """
         func_name = "ls"
         if self.help_function_child_name in (func_name, "dir"):
@@ -1397,7 +1261,7 @@ Output:
         self.current_tty_status = status
         return status
 
-    def hello_world(self, args: list) -> int:
+    def hello_world(self, args: List) -> int:
         """ This is a function in charge of displaying a Hello World and the passed arguments """
         func_name = "hello_world"
         if self.help_function_child_name == func_name:
@@ -1421,7 +1285,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def run_command(self, args: list) -> int:
+    def run_command(self, args: List) -> int:
         """ Run a command in the host's shell environement """
         help_command = "run"
         if self.help_function_child_name == help_command:
@@ -1493,6 +1357,10 @@ if %errorLevel% == 0 (
 
     def run_as_windows_admin(self, file: str) -> int:
         """ Run a powershell script as an administrator """
+        self.print_on_tty(
+            self.info_colour,
+            "!! As of date, due to the complexity of running a command with elevated privileges, the execution status cannot be properly tracked !!\n"
+        )
         return self.run_command(
             [
                 "powershell.exe",
@@ -1506,7 +1374,7 @@ if %errorLevel% == 0 (
             ]
         )
 
-    def check_admin(self, args: list) -> int:
+    def check_admin(self, args: List) -> int:
         """ Check if the program has admin rights """
         func_name = "check_admin"
         if self.help_function_child_name == func_name:
@@ -1552,7 +1420,12 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def run_as_admin(self, args: list) -> int:
+    def _set_file_content(self, file_path: str, content: str, mode: str = "w", encoding: str = "utf-8", newline: str = "\n") -> int:
+        with open(file_path, mode, encoding=encoding, newline=newline) as file:
+            file.write(content)
+        return self.success
+
+    def run_as_admin(self, args: List) -> int:
         """ Run a command as an administrator """
         func_name = "run_as_admin"
         if self.help_function_child_name == func_name:
@@ -1594,7 +1467,37 @@ Output:
                 return status
             self.current_tty_status = self.success
             return self.success
-        return self.run_command(["sudo", args])
+        file_name = "/tmp/your_code.sh"
+        user_input = " ".join(args)
+        self._set_file_content(
+            file_path=file_name,
+            content=user_input,
+            mode="w",
+            encoding="utf-8",
+            newline="\n"
+        )
+        new_command = [
+            "chmod",
+            "777",
+            file_name,
+            "&&",
+            "sudo",
+            "su",
+            "-c",
+            file_name
+        ]
+        return self.run_command(new_command)
+
+    def is_exactly_in_string(self, string1: str, string2: str) -> bool:
+        """ Check if string1 is exactly in string2 """
+        string1_length = string1
+        string2_length = string2
+        if string1_length != string2_length:
+            return False
+        for index, item in enumerate(string1):
+            if item != string2[index]:
+                return False
+        return True
 
     def process_input(self) -> None:
         """ The function in charge of processing the user input """
@@ -1608,11 +1511,15 @@ Output:
         command = command[0].lower()
         was_found = False
         for item in self.options:
-            if command in item:
+            if self.is_exactly_in_string(list(item)[0], command) is True:
                 item[command](args)
                 was_found = True
+                break
         if was_found is False:
-            self.print_on_tty(self.error_colour, "Invalid option\n")
+            self.print_on_tty(
+                self.error_colour,
+                f"Invalid option: {str(command)}\n"
+            )
             self.current_tty_status = self.err
 
     def assing_colours(self) -> None:
@@ -1649,7 +1556,7 @@ Output:
         self.env_definition_colour = self.tty_colours["env_definition_colour"]
         self.session_name_colour = self.tty_colours["session_name_colour"]
 
-    def command_seperator(self, args: list) -> int:
+    def command_seperator(self, args: List) -> int:
         """ Display/Change the token in charge of indicating the beginning of a new command when many are put together """
         func_name = "command_seperator"
         if self.help_function_child_name == func_name:
@@ -1710,7 +1617,7 @@ Output:
         self.current_tty_status = self.success
         return self.success
 
-    def update_comment_token(self, args: list) -> int:
+    def update_comment_token(self, args: List) -> int:
         """ Display/Change the token in charge of indicating the beginning of a new comment """
         func_name = "comment_token"
         if self.help_function_child_name == func_name:
@@ -1794,6 +1701,12 @@ Output:
         else:
             self.home = os.getcwd()
 
+    def commands_to_auto_complete(self) -> None:
+        """ Convert the available commands to a list so that it can be used for command auto-completion """
+        for i in self.options:
+            for b in i:
+                self.auto_complete_list.append(b)
+
     def load_basics(self) -> None:
         """ set the values for the variables that can be configured by the user """
         self.old_pwd = os.getcwd()
@@ -1801,85 +1714,136 @@ Output:
         self.assing_colours()
         self.colour_lib.init_pallet()
         self.options = [
-            {"help": self.help, "desc": "Display this help section"},
-            {"man": self.help, "desc": "Display this help section"},
-            {".help": self.help, "desc": "Display this help section"},
-            {".h": self.help, "desc": "Display this help section"},
-            {"/?": self.help, "desc": "Display this help section"},
-            {"-h": self.help, "desc": "Display this help section"},
-            {"--h": self.help, "desc": "Display this help section"},
-            {"-help": self.help, "desc": "Display this help section"},
-            {"--help": self.help, "desc": "Display this help section"},
-            {"setenv": self.setenv, "desc": "Set a variable in the environement"},
+            {
+                "help": self.help,
+                self.command_description_token_inner: "Display this help section"
+            },
+            {
+                "man": self.help,
+                self.command_description_token_inner: "Display this help section"
+            },
+            {
+                ".help": self.help,
+                self.command_description_token_inner: "Display this help section"
+            },
+            {".h": self.help, self.command_description_token_inner: "Display this help section"},
+            {"/?": self.help, self.command_description_token_inner: "Display this help section"},
+            {"-h": self.help, self.command_description_token_inner: "Display this help section"},
+            {
+                "--h": self.help,
+                self.command_description_token_inner: "Display this help section"
+            },
+            {
+                "-help": self.help,
+                self.command_description_token_inner: "Display this help section"
+            },
+            {
+                "--help": self.help,
+                self.command_description_token_inner: "Display this help section"
+            },
+            {
+                "setenv": self.setenv,
+                self.command_description_token_inner: "Set a variable in the environement"
+            },
             {
                 "unsetenv": self.unsetenv,
-                "desc": "Remove a variable from the environement"
+                self.command_description_token_inner: "Remove a variable from the environement"
+            },
+            {
+                "exit": self.exit,
+                self.command_description_token_inner: "Close the current menu"
             },
-            {"exit": self.exit, "desc": "Close the current menu"},
             {
                 "abort": self.kill,
-                "desc": "Exit the program (This will kill the program and any child processes)"
+                self.command_description_token_inner: "Exit the program (This will kill the program and any child processes)"
             },
-            {"hello_world": self.hello_world, "desc": "Display a Hello World"},
-            {"env": self.env, "desc": "Display the environement variables"},
+            {
+                "hello_world": self.hello_world,
+                self.command_description_token_inner: "Display a Hello World"
+            },
+            {"env": self.env, self.command_description_token_inner: "Display the environement variables"},
             {
                 "env++": self.env_plus_plus,
-                "desc": "Display the environement variables using different colours"
+                self.command_description_token_inner: "Display the environement variables using different colours"
             },
             {
                 "?": self.display_status_code,
-                "desc": "Display the status code of the last function called"
+                self.command_description_token_inner: "Display the status code of the last function called"
+            },
+            {
+                "cd": self.change_directory,
+                self.command_description_token_inner: "Change the current working directory"
+            },
+            {
+                "pwd": self.pwd, self.command_description_token_inner:
+                "Display the path to the directory in wich we are located"
             },
-            {"cd": self.change_directory,
-                "desc": "Change the current working directory"},
-            {"pwd": self.pwd, "desc": "Display the path to the directory in wich we are located"},
             {
                 "version": self.version,
-                "desc": "Display the current version of the program"
+                self.command_description_token_inner: "Display the current version of the program"
+            },
+            {
+                "author": self.author,
+                self.command_description_token_inner: "Display the author of the program"
             },
-            {"author": self.author, "desc": "Display the author of the program"},
             {
                 "session_name": self.process_session_name,
-                "desc": "Change the name of the current session"
+                self.command_description_token_inner: "Change the name of the current session"
+            },
+            {
+                "client": self.client,
+                self.command_description_token_inner: "Display the client of the program"
             },
-            {"client": self.client, "desc": "Display the client of the program"},
             {
                 "history": self.show_history,
-                "desc": "Display the previous commands that were run"
+                self.command_description_token_inner: "Display the previous commands that were run"
+            },
+            {
+                "ls": self.bind_ls, self.command_description_token_inner:
+                "List all files in the current folder"
+            },
+            {
+                "dir": self.bind_ls, self.command_description_token_inner:
+                "List all files in the current folder"
             },
-            {"ls": self.bind_ls, "desc": "List all files in the current folder"},
-            {"dir": self.bind_ls, "desc": "List all files in the current folder"},
             {
                 "mkdir": self.make_directory,
-                "desc": "Create a directory in the present path"
+                self.command_description_token_inner: "Create a directory in the present path"
+            },
+            {
+                "touch": self.touch,
+                self.command_description_token_inner: "Create a file in the present path"
             },
-            {"touch": self.touch, "desc": "Create a file in the present path"},
             {
                 "rm": self.remove_file,
-                "desc": "Remove a file or directory if present in the path"
+                self.command_description_token_inner: "Remove a file or directory if present in the path"
             },
             {
                 "rmdir": self.remove_directory,
-                "desc": "Remove a directory if present in the path"
+                self.command_description_token_inner: "Remove a directory if present in the path"
+            },
+            {
+                "run": self.run_command,
+                self.command_description_token_inner: "Run a command in the system terminal"
             },
-            {"run": self.run_command, "desc": "Run a command in the system terminal"},
             {
                 "is_admin": self.check_admin,
-                "desc": "Return True if the system has elevated privileges."
+                self.command_description_token_inner: "Return True if the system has elevated privileges."
             },
             {
                 "super_run": self.run_as_admin,
-                "desc": "Run a command using elevated privileges"
+                self.command_description_token_inner: "Run a command using elevated privileges"
             },
             {
                 "command_seperator": self.command_seperator,
-                "desc": "Display/Change the token in charge of indicating the begining of a new command when many are put together"
+                self.command_description_token_inner: "Display/Change the token in charge of indicating the beginning of a new command when many are put together"
             },
             {
                 "comment_token": self.update_comment_token,
-                "desc": "Display/Change the token in charge of indicating the begining of a new command when many are put together"
+                self.command_description_token_inner: "Display/Change the token in charge of indicating the beginning of a new command when many are put together"
             }
         ]
+        self.commands_to_auto_complete()
 
     def unload_basics(self) -> int:
         """ Free the ressources that were previously allocated """
@@ -1900,29 +1864,33 @@ Output:
         self.session_name_colour = None
         self.tty_colours = None
         self.options = []
+        self.auto_complete_list = []
         return self.colour_lib.unload_ressources()
 
-    def import_functions_into_shell(self, functions: list[dict[str, any]]) -> int:
+    def import_functions_into_shell(self, functions: List[Dict[str, any]]) -> int:
         """ Import functions into the shell """
         for function in functions:
-            if function is None or isinstance(function, dict) != True:
+            if function is None or isinstance(function, Dict) != True:
                 continue
             if "desc" not in function:
                 function["desc"] = "No description provided\n"
             self.options.append(function)
+            item = list(function)[0]
+            self.auto_complete_list.append(item)
             self.print_on_tty(
                 self.success_colour,
-                f"Added function {list(function)[0]}\n"
+                f"Added function {item}\n"
             )
         self.current_tty_status = self.success
         return self.success
 
-    def remove_function_from_options(self, function: dict[str, any]) -> int:
+    def remove_function_from_options(self, function: Dict[str, any]) -> int:
         """ Remove a function from the options """
         for function_item in self.options:
             for index, item in enumerate(function_item):
                 if item == function:
                     self.options.pop(index)
+                    self.auto_complete_list.pop(index)
                     self.print_on_tty(self.success, f"Removed function {item}")
                     self.current_tty_status = self.success
                     return self.current_tty_status
@@ -1933,7 +1901,7 @@ Output:
         self.current_tty_status = self.error
         return self.current_tty_status
 
-    def remove_functions_from_shell(self, functions: list[dict[str, any]]) -> int:
+    def remove_functions_from_shell(self, functions: List[Dict[str, any]]) -> int:
         """ Remove functions from the shell """
         global_status = self.success
         for function in functions:
@@ -1954,13 +1922,13 @@ Output:
         else:
             self.print_on_tty(self.error_colour, goodbye_message)
 
-    def run_complex_input(self, complex_input: list[str]) -> None:
+    def run_complex_input(self, complex_input: List[str]) -> None:
         """ Run a complex input """
         for item in complex_input:
             self.user_input = item
             self.process_input()
 
-    def process_complex_input(self, usr_input: list) -> None:
+    def process_complex_input(self, usr_input: List) -> None:
         """ process multiple command input if provided """
         command_list = []
         buffer = ""
@@ -1996,10 +1964,24 @@ Output:
                     "\n\n\n\n"
                 )
 
+    def clean_string(self, input_string: str) -> str:
+        """ remove enclosing string from the run string """
+        if len(input_string) > 0:
+            if input_string[0] == '"':
+                input_string = input_string[1:]
+            if input_string[-1] == '"':
+                input_string = input_string[:-1]
+            if input_string[-2] == '"':
+                input_string = input_string[:-2]+input_string[-1:]
+            if input_string[-3] == '"':
+                input_string = input_string[:-3]+input_string[-2:]
+        return input_string
+
     def process_if_pipe_input(self) -> None:
         """ Check if the user input is a pipe input """
         if not sys.stdin.isatty():
             user_input = sys.stdin.read()
+            user_input = self.clean_string(user_input)
             user_args = user_input.split(self.input_split_char)
             self.process_complex_input(user_args)
             if self.continue_tty_loop is True:
